/*
 *      print.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "listProducts.h"
#include "interface.h"
#include "callbacks.h"
#include "config.h"
#include "preferences.h"
#include "print.h"

/********************************** Interface **********************************/

void create_pPrint(glda *pGlda){
  Print *pPrintData;
  GtkPrintOperation *operation;
  GtkPrintSettings *settings = NULL;
  GError *error = NULL;
  gint res;
  
  pPrintData = (Print*)malloc(sizeof(Print));
  pPrintData->pGlda = pGlda;
  
  operation = gtk_print_operation_new();
  
  if(settings != NULL)
    gtk_print_operation_set_print_settings(operation, settings);
  
  g_signal_connect(G_OBJECT(operation), "begin_print", G_CALLBACK(begin_print), (Print*)pPrintData);
  g_signal_connect(G_OBJECT(operation), "draw_page", G_CALLBACK(draw_page), (Print*)pPrintData);
  g_signal_connect(G_OBJECT(operation), "end_print", G_CALLBACK(end_print), (Print*)pPrintData);
  
  res = gtk_print_operation_run(operation, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
				GTK_WINDOW(pGlda->pWindow), &error);
  
  if(res == GTK_PRINT_OPERATION_RESULT_APPLY){
    settings = g_object_ref(gtk_print_operation_get_print_settings(operation));
  }
  
  g_object_unref(operation);
  free(pPrintData);
}

/********************************** Callbacks **********************************/

void begin_print(GtkPrintOperation *operation, GtkPrintContext *context, Print *pPrintData){
	gdouble height;
  gdouble size_police;
  GtkTreeIter Iter;
  
  /* police's size */
  size_police = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "size_police", NULL);
  
  /* page's height */
  height = gtk_print_context_get_height(context) - HEADER_HEIGHT - HEADER_GAP - 2 * size_police;
  
	pPrintData->number_products_per_page = floor(height / (2 * size_police));
	
	/* nombre d'articles */
  pPrintData->number_products = pPrintData->pGlda->nbTotalProducts;
  
  pPrintData->number_products_differents = 0;
  if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &Iter)){
  	while(gtk_tree_model_iter_next(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &Iter)){
  		pPrintData->number_products_differents++;
  	}
  }
  
  /* number of pages */
  pPrintData->total_pages = pPrintData->number_products_differents / pPrintData->number_products_per_page + 1;
  
  gtk_print_operation_set_n_pages(operation, pPrintData->total_pages);
}

void draw_page(GtkPrintOperation *operation, GtkPrintContext *context, gint page_nr, Print *pPrintData){
	cairo_t *cr;
  PangoLayout *layout;
  gdouble width, text_height;
  gint number_products, i, text_width, layout_height;
  PangoFontDescription *desc;
  gchar *page_str;
  gchar *pText = NULL;
  gchar *pName = NULL;
  gchar *pPrice = NULL;
  gchar *pQuantity = NULL;
  //gint quantity;
  gint length_line, length_maxi_number, length_maxi_name, length_maxi_price, length_maxi_quantity, cellpadding;
  gdouble size_police, border;
  gboolean show_title, show_page, show_column_number, show_column_name, show_column_price, show_column_quantity, show_line_total;
  gboolean b;
  gdouble *color;
  gsize length;
  GdkColor gdkColor;
  gchar *rgb;
  
  cr = gtk_print_context_get_cairo_context(context);
  width = gtk_print_context_get_width(context);
  layout = gtk_print_context_create_pango_layout(context);
  size_police = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "size_police", NULL);
  desc = pango_font_description_from_string(g_key_file_get_string(pPrintData->pGlda->pConfig, "print", "police", NULL));
  pango_font_description_set_size(desc, size_police * PANGO_SCALE);

  pango_layout_get_size(layout, NULL, &layout_height);
  text_height = (gdouble)layout_height / PANGO_SCALE;
    
  pango_layout_set_font_description(layout, desc);
  
  show_title = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_title", NULL);
  show_page = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_page", NULL);
  color = g_key_file_get_double_list(pPrintData->pGlda->pConfig, "print", "color", &length, NULL);
  border = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "border", NULL);
  show_column_number = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_number", NULL);
  show_column_name = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_name", NULL);
  show_column_price = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_price", NULL);
  show_column_quantity = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_quantity", NULL);
  length_maxi_number = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_number", NULL);
  length_maxi_name = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_name", NULL);
  length_maxi_price = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_price", NULL);
  length_maxi_quantity = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_quantity", NULL);
  show_line_total = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_line_total", NULL);
  cellpadding = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "cellpadding", NULL);
  
   /* set color */
  //cairo_set_source_rgb(cr, color[0], color[1], color[2]);
  rgb = g_strdup_printf("#%02x%02x%02x", (gint)color[0], (gint)color[1], (gint)color[2]);
	gdk_color_parse(rgb, &gdkColor);
  gdk_cairo_set_source_color(cr, &gdkColor);
  g_free(rgb);
  
  /* set line's size */
	cairo_set_line_width(cr, border);
	
  /* title */
  if(show_title){
	  pText = g_strdup_printf("Glda version %s", VERSION_GLDA);
  	pango_layout_set_text(layout, pText, -1);
  	g_free(pText);
  	pango_layout_set_width(layout, -1);
  	pango_layout_set_alignment(layout, PANGO_ALIGN_LEFT);
  
  	cairo_move_to(cr, 0, (HEADER_HEIGHT - text_height) / 2);
  	pango_cairo_show_layout(cr, layout);
  }
  
  /* page */
  if(show_page){
	  page_str = g_strdup_printf("%d / %d", page_nr+1, pPrintData->total_pages);
  	pango_layout_set_text(layout, page_str, -1);
  	pango_layout_get_size(layout, &text_width, NULL);
  	pango_layout_set_alignment(layout, PANGO_ALIGN_RIGHT);
  	g_free(page_str);
  	
  	cairo_move_to(cr, width - (text_width / PANGO_SCALE), (HEADER_HEIGHT - text_height) / 2);
  	pango_cairo_show_layout(cr, layout);
  }
	
	/* length line */
	length_line = 0;
	if(show_column_number)
		length_line = length_line + length_maxi_number + 1;
	if(show_column_name)
		length_line = length_line + length_maxi_name + 1;
	if(show_column_price)
		length_line = length_line + length_maxi_price + 1;
	if(show_column_quantity)
		length_line = length_line + length_maxi_quantity + 1;

	/* First line */
	cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP);
	cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);

  cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP);
	cairo_line_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	
	length_line = 0;
	
	/* colunm number */
	if(show_column_number){
		cairo_move_to(cr, size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "N°", length_maxi_number);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_number + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* column product's name */
	if(show_column_name){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Article(s)", length_maxi_name);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_name + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* column product's price */
	if(show_column_price){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Prix", length_maxi_price);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_price + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* column quantity */
	if(show_column_quantity){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Quantité", length_maxi_quantity);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_quantity + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* line */
	cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	
  number_products = page_nr * pPrintData->number_products_per_page;
  i=0;
  if(page_nr == 0){
  	b = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &(pPrintData->Iter));
  }else{
  	b = gtk_tree_model_iter_next(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &(pPrintData->Iter));
  }
  if(b){
  	do{
  		gtk_tree_model_get(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &(pPrintData->Iter),
																				PRODUCTS_ADDED_COLUMN, &pName,
																				PRICE_PRODUCTS_ADDED_COLUMN, &pPrice,
																				QUANTITY_PRODUCTS_ADDED_COLUMN, &pQuantity/*&quantity*/,
																				-1);
			
  		length_line = 0;
  		
    	/* First column line */
	    cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			
			/* column number */
			if(show_column_number){
  	  	cairo_move_to(cr, size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
	  	  pText = g_strdup_printf("%d", number_products+1);
				pango_layout_set_text(layout, pText, length_maxi_number);
				g_free(pText);
				pango_cairo_show_layout(cr, layout);
				
				length_line = length_line + length_maxi_number + 1;
				
				cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
				cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			}
		
			/* column product's name */
			if(show_column_name){
				cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
				pango_layout_set_text(layout, pName, length_maxi_name);
				pango_cairo_show_layout(cr, layout);
				
				length_line = length_line + length_maxi_name + 1;
				
				cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
				cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			}
		
			/* column product's price */
			if(show_column_price){
				cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
				pango_layout_set_text(layout, pPrice, length_maxi_price);
				pango_cairo_show_layout(cr, layout);
				
				length_line = length_line + length_maxi_price + 1;
				
				cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
				cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			}
		
			/* column quantity */
			if(show_column_quantity){
				cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
				//pQuantity = g_strdup_printf("%.2f", quantity);
				pango_layout_set_text(layout, pQuantity, length_maxi_quantity);
				pango_cairo_show_layout(cr, layout);
				
				length_line = length_line + length_maxi_quantity + 1;
				
				cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
				cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			}
		
			cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
			
			number_products++;
			i++;
			g_free(pName);
			g_free(pPrice);
			g_free(pQuantity);
			pName = NULL;
			pPrice = NULL;
			pQuantity = NULL;
		}while(gtk_tree_model_iter_next(GTK_TREE_MODEL(pPrintData->pGlda->pTreeStoreProductsAdded), &(pPrintData->Iter))
							&& i<pPrintData->number_products_per_page);
  }
  
  /* Total */
  if(page_nr == pPrintData->total_pages - 1 && show_line_total){
	  gint start_total;
  	length_line = 0;
  	
  	if(show_column_number){
  		length_line = length_line + length_maxi_number + 1;
  	}
  	
  	start_total = length_line;
  	
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		
		if(show_column_name){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pango_layout_set_text(layout, "Total", length_maxi_name);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_name + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		if(show_column_price){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%4.2f %s", pPrintData->pGlda->priceTotalProducts, g_key_file_get_string(pPrintData->pGlda->pConfig, "general", "devise", NULL));
			pango_layout_set_text(layout, pText, length_maxi_price);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_price + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		if(show_column_quantity){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%.2f", pPrintData->pGlda->nbTotalProducts);
			pango_layout_set_text(layout, pText, length_maxi_quantity);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_quantity + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		cairo_move_to(cr, start_total * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
	}
	
  cairo_stroke(cr);
  
  g_object_unref(layout);
  pango_font_description_free(desc);
}

void end_print(GtkPrintOperation *operation, GtkPrintContext *context, Print *pPrintData){
	
}
